"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionManager = void 0;
const instancefactory_1 = require("../main/instancefactory");
const aopfactory_1 = require("../main/aopfactory");
const transactionadvice_1 = require("./transactionadvice");
const errorfactory_1 = require("../tools/errorfactory");
const mysqltransaction_1 = require("./mysqltransaction");
const dbmanager_1 = require("./dbmanager");
const oracletransaction_1 = require("./oracletransaction");
const application_1 = require("../tools/application");
const mssqltransaction_1 = require("./mssqltransaction");
const typeormtransaction_1 = require("./typeormtransaction");
const relaentransaction_1 = require("./relaentransaction");
const threadlocal_1 = require("../tools/threadlocal");
class TransactionManager {
    static init(cfg) {
        //transaction 模块实例名
        this.transactionMdl = cfg.transaction || 'noomi_transaction';
        //隔离级
        if (cfg.isolation_level && typeof cfg.isolation_level === 'number') {
            this.isolationLevel = cfg.isolation_level;
        }
        //添加Aspect
        let adviceInstance = instancefactory_1.InstanceFactory.addInstance({
            name: 'NoomiTransactionAdvice',
            instance: new transactionadvice_1.TransactionAdvice(),
            class: transactionadvice_1.TransactionAdvice
        });
        aopfactory_1.AopFactory.addPointcut(this.pointcutId, []);
        //增加pointcut expression
        process.nextTick(() => {
            aopfactory_1.AopFactory.addExpression(TransactionManager.pointcutId, TransactionManager.addToAopExpressions);
        });
        //增加advice
        aopfactory_1.AopFactory.addAdvice({
            pointcut_id: this.pointcutId,
            type: 'before',
            method: 'before',
            instance: adviceInstance
        });
        aopfactory_1.AopFactory.addAdvice({
            pointcut_id: this.pointcutId,
            type: 'after-return',
            method: 'afterReturn',
            instance: adviceInstance
        });
        aopfactory_1.AopFactory.addAdvice({
            pointcut_id: this.pointcutId,
            type: 'after-throw',
            method: 'afterThrow',
            instance: adviceInstance
        });
        //添加transaction到实例工厂，已存在则不再添加
        let tn = this.transactionMdl;
        //事务类
        let clazz;
        if (tn) {
            let ins = instancefactory_1.InstanceFactory.getInstance(tn);
            if (ins === null) {
                switch (cfg.product) {
                    case "relaen":
                        clazz = relaentransaction_1.RelaenTransaction;
                        break;
                    case "mysql":
                        clazz = mysqltransaction_1.MysqlTransaction;
                        break;
                        break;
                    case "mssql":
                        clazz = mssqltransaction_1.MssqlTransaction;
                        break;
                    case "oracle":
                        clazz = oracletransaction_1.OracleTransaction;
                        break;
                    case 'typeorm': //typeorm
                        clazz = typeormtransaction_1.TypeormTransaction;
                        this.transactionOption = {};
                        //设置隔离级别
                        if (this.isolationLevel !== 0) {
                            switch (TransactionManager.isolationLevel) {
                                case 1:
                                    this.transactionOption.isolationLevel = "READ UNCOMMITTED";
                                    break;
                                case 2:
                                    this.transactionOption.isolationLevel = "READ COMMITTED";
                                    break;
                                case 3:
                                    this.transactionOption.isolationLevel = "REPEATABLE READ";
                                    break;
                                case 4:
                                    this.transactionOption.isolationLevel = "SERIALIZABLE";
                            }
                        }
                        break;
                }
                instancefactory_1.InstanceFactory.addInstance({
                    name: tn,
                    class: clazz,
                    singleton: false
                });
            }
        }
    }
    /**
     * 添加为事务
     * @param instance      实例 或 类
     * @param methodName    方法名
     */
    static addTransaction(instance, methodName) {
        let pc = aopfactory_1.AopFactory.getPointcutById(this.pointcutId);
        let name = typeof instance === 'string' ? instance : instance.__instanceName;
        let expr = '^' + name + '.' + methodName + '$';
        if (pc) { //pointcut存在，直接加入表达式
            aopfactory_1.AopFactory.addExpression(this.pointcutId, expr);
        }
        else { //pointcut不存在，加入待处理队列
            this.addToAopExpressions.push(expr);
        }
    }
    /**
     * 获取transaction
     * @param newOne    如果不存在，是否新建
     * @return          transacton
     */
    static async get(newOne) {
        let tr;
        //得到当前执行异步id
        let id = threadlocal_1.NoomiThreadLocal.getThreadId();
        if (!id && newOne) {
            id = threadlocal_1.NoomiThreadLocal.newThreadId();
        }
        if (!id) {
            return null;
        }
        if (this.transactionMap.has(id)) {
            tr = this.transactionMap.get(id);
        }
        else if (newOne) { //父亲对象不存在，则新建
            tr = instancefactory_1.InstanceFactory.getInstance(this.transactionMdl, [id]);
            // 保存到transaction map
            this.transactionMap.set(id, tr);
        }
        return tr;
    }
    /**
     * 删除事务
     * @param tr    事务
     */
    static del(tr) {
        this.transactionMap.delete(tr.id);
    }
    /**
     * 获取connection
     */
    static getConnection(id) {
        if (!id) {
            id = threadlocal_1.NoomiThreadLocal.getThreadId();
        }
        if (!this.transactionMap.has(id)) {
            return null;
        }
        let tr = this.transactionMap.get(id);
        return tr.connection;
    }
    /**
     * 释放连接
     * @param tr
     */
    static async releaseConnection(tr) {
        await dbmanager_1.DBManager.getConnectionManager().release(tr.connection);
    }
    /**
     * 解析实例配置文件
     * @param path      文件路径
     * @param mdlPath   模型路径
     */
    static parseFile(path) {
        //读取文件
        let jsonStr = application_1.App.fs.readFileSync(path, 'utf-8');
        let json = null;
        try {
            json = application_1.App.JSON.parse(jsonStr);
        }
        catch (e) {
            throw new errorfactory_1.NoomiError("2800") + '\n' + e;
        }
        this.init(json);
    }
}
exports.TransactionManager = TransactionManager;
TransactionManager.transactionMap = new Map(); //transaction map
TransactionManager.pointcutId = 'NOOMI_TX_POINTCUT'; //切点名
TransactionManager.addToAopExpressions = []; //待添加到transaction aop的表达式串
TransactionManager.isolationLevel = 0; //隔离级 1read uncommited 2read commited 3repeatable read 4serializable
//# sourceMappingURL=transactionmanager.js.map